package cn.com.qmth.examcloud.core.examwork.api.provider;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgSettingsRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgSettingsEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamInfo;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamServiceImpl;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamPropertyReq;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.GetOngoingExamListReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SetExamPropertyReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamPropertyResp;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.examwork.api.response.GetOngoingExamListResp;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamResp;
import cn.com.qmth.examcloud.examwork.api.response.SetExamPropertyResp;
import io.swagger.annotations.ApiOperation;

/**
 * @author chenken
 * @date 2018年5月3日 下午2:08:59
 * @company QMTH
 * @description ExamProvider.java
 */
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.examwork}" + "exam")
public class ExamCloudServiceProvider extends ControllerSupport implements ExamCloudService {

	private static final long serialVersionUID = 6850508158980856785L;

	@Autowired
	private ExamServiceImpl examService;

	@Autowired
	ExamOrgSettingsRepo examOrgSettingsRepo;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamPropertyRepo examPropertyRepo;

	@ApiOperation(value = "新增考试批次", notes = "新增")
	@PostMapping("saveExam")
	@Override
	public SaveExamResp saveExam(@RequestBody SaveExamReq req) {
		ExamInfo examInfo = new ExamInfo();

		examInfo.setBeginTime(req.getBeginTime());
		examInfo.setDuration(req.getDuration());
		examInfo.setEnable(true);
		examInfo.setEndTime(req.getEndTime());
		examInfo.setExamTimes(req.getExamTimes());

		ExamType et = null;
		try {
			et = ExamType.valueOf(req.getExamType());
		} catch (Exception e) {
			throw new StatusException("E-001004", "考试类型错误");
		}

		examInfo.setExamType(et);
		examInfo.setId(req.getId());
		examInfo.setName(req.getName());
		examInfo.setRemark(req.getRemark());
		examInfo.setRootOrgId(req.getRootOrgId());

		Map<String, String> properties = req.getProperties();
		examInfo.setProperties(properties);

		ExamEntity saved = examService.saveExam(examInfo);

		SaveExamResp resp = new SaveExamResp();
		resp.setExamId(saved.getId());
		return resp;
	}

	@ApiOperation(value = "查询考试")
	@PostMapping("getExam")
	@Override
	public GetExamResp getExam(@RequestBody GetExamReq req) {
		Long id = req.getId();
		Long rootOrgId = req.getRootOrgId();
		String name = req.getName();
		if (null == id && StringUtils.isBlank(name)) {
			throw new StatusException("E-002002", "id,name不能都为空");
		}
		if (null != id && StringUtils.isNotBlank(name)) {
			throw new StatusException("E-002003", "id,name不能都不为空");
		}
		ExamEntity exam = null;

		if (null != id) {
			exam = examRepo.findOne(id);
		} else if (StringUtils.isNotBlank(name)) {
			if (null == rootOrgId) {
				throw new StatusException("E-002004", "rootOrgId is null");
			}
			exam = examService.findExamByNameAndRootOrgId(name, rootOrgId);
		}

		if (null == exam) {
			throw new StatusException("E-002005", "考试不存在");
		}
		GetExamResp examResp = new GetExamResp();
		examResp.setId(exam.getId());
		ExamBean bean = new ExamBean();
		examResp.setExamBean(bean);

		bean.setId(exam.getId());
		bean.setBeginTime(exam.getBeginTime());
		bean.setDuration(exam.getDuration());
		bean.setEnable(exam.getEnable());
		bean.setEndTime(exam.getEndTime());
		bean.setExamTimes(exam.getExamTimes());
		bean.setExamType(exam.getExamType().name());
		bean.setName(exam.getName());
		bean.setRemark(exam.getRemark());
		bean.setRootOrgId(exam.getRootOrgId());

		Long orgId = req.getOrgId();
		if (null != orgId) {
			ExamOrgSettingsEntity examOrgEntity = examOrgSettingsRepo
					.findByExamIdAndOrgId(exam.getId(), orgId);

			bean.setBeginTime(examOrgEntity.getBeginTime());
			bean.setEndTime(examOrgEntity.getEndTime());
		}

		return examResp;
	}

	@ApiOperation(value = "设置考试属性")
	@PostMapping("setExamProperty")
	@Override
	public SetExamPropertyResp setExamProperty(@RequestBody SetExamPropertyReq req) {
		Long examId = req.getExamId();
		String key = req.getKey();
		String value = req.getValue();

		ExamProperty prop = ExamProperty.valueOf(key);

		ExamPropertyEntity entity = examPropertyRepo.findByexamIdAndKeyId(examId, prop.getKeyId());
		if (null == entity) {
			entity = new ExamPropertyEntity();
			entity.setExamId(examId);
			entity.setKeyId(prop.getKeyId());
		}
		entity.setValue(value);

		ExamPropertyEntity saved = examPropertyRepo.save(entity);

		SetExamPropertyResp resp = new SetExamPropertyResp();
		resp.setPropertyId(saved.getId());
		return resp;
	}

	@ApiOperation(value = "查询考试属性")
	@PostMapping("getExamProperty")
	@Override
	public GetExamPropertyResp getExamProperty(@RequestBody GetExamPropertyReq req) {
		Long examId = req.getExamId();
		Long orgId = req.getOrgId();
		String key = req.getKey();

		String value = examService.getOrgProperty(examId, orgId, key);

		GetExamPropertyResp resp = new GetExamPropertyResp();
		resp.setValue(value);
		return resp;
	}

	@Override
	public GetOngoingExamListResp getOngoingExamList(GetOngoingExamListReq req) {

		Long rootOrgId = req.getRootOrgId();

		Specification<ExamEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));
			predicates.add(cb.lessThan(root.get("beginTime"), new Date()));
			predicates.add(cb.greaterThan(root.get("endTime"), new Date()));

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(0, 100, new Sort(Direction.DESC, "updateTime"));

		Page<ExamEntity> page = examRepo.findAll(specification, pageRequest);

		Iterator<ExamEntity> iterator = page.iterator();

		List<ExamBean> list = Lists.newArrayList();

		while (iterator.hasNext()) {
			ExamEntity exam = iterator.next();
			ExamBean bean = new ExamBean();
			list.add(bean);

			bean.setId(exam.getId());
			bean.setBeginTime(exam.getBeginTime());
			bean.setDuration(exam.getDuration());
			bean.setEnable(exam.getEnable());
			bean.setEndTime(exam.getEndTime());
			bean.setExamTimes(exam.getExamTimes());
			bean.setExamType(exam.getExamType().name());
			bean.setName(exam.getName());
			bean.setRemark(exam.getRemark());
			bean.setRootOrgId(exam.getRootOrgId());
		}
		GetOngoingExamListResp resp = new GetOngoingExamListResp();
		resp.setExamList(list);
		return resp;
	}

}
